@echo off
REM GRID_SCRIPT_VERSION:300
REM /---------------------------------------------------------------------\
REM |                                                                     |
REM |     Script for starting remote SAS/CONNECT server in a Grid         |
REM |                                                                     |
REM | NOTE: This file must be saved with CR/LF termination for the code   |
REM |       to work properly. If not you will get missing label errors.   |
REM |                                                                     |
REM | NOTE: Environment variables set or used inside a if ( ) block       |
REM |       does not expand because they are evaluated at load time not   |
REM |       run time. This is why there are so many goto's in the script  |
REM |                                                                     |
REM \---------------------------------------------------------------------/

REM /-----------------------------------------------------------
REM | Since the grid provider may copy over environments,
REM | remove the METAUSER/METAPASS values since they may
REM | contain a bad value causing connections to the
REM | metadata server to fail. Similarly, if a SAS session
REM | is spawned from a SAS/CONNECT spawner and that session
REM | starts a grid job, the INHERIT environment variable
REM | needs to be reset to 0 to indicate it is not inheriting
REM | a socket connection.
REM \-----------------------------------------------------------
set METAUSER=
set METAPASS=
set INHERIT=0

REM /-----------------------------------------------------------
REM | Save the path to the script file to look for app server
REM | We must do this before we start shifting parms
REM \-----------------------------------------------------------
set SASGRIDCMDPATH=%~dp0

REM /-----------------------------------------------------------
REM | Load in the information about the install from the same
REM | location that this script was found.
REM \-----------------------------------------------------------
if exist "%SASGRIDCMDPATH%\grid.cfg.cmd" (
    call "%SASGRIDCMDPATH%\grid.cfg.cmd"
)

REM /-----------------------------------------------------------
REM | Reset the environment variables used
REM \-----------------------------------------------------------
set RUNMODE=DMR
set SASDAEMONHOST=
set SASDAEMONPORT=
set SASCLIENTPORT=
set SASARM4CORR=
set SASUSERARGS=
set SASCFGPATH=
set SASLOGCFGFILE=
set SASCHKPTRESTART=
set SASGRIDWORKDIR=
set SASGRIDBATCHPGM=
set SASWANTJOBINFO=
set SASMPIPROCESSES=
set SASTKMPICLIENT=
set SASTKMPICPORT=
set SASTKMPICTKLIB=
set tempvar1=
set tempvar2=
set tempvar3=

REM /-----------------------------------------------------------
REM | Set the passed in parameters into the environment
REM \-----------------------------------------------------------
:parseloop

  if '%1'=='' goto exitloop

REM -- Set the environment variable 'value' to the parm minus the double quotes.

  set value=%~1

REM -- Parse the name from the name:value pair
REM
REM    /I does case insensitive compare
REM
REM    %value:~0,13% means "take the substring starting
REM    at position 0 to position 13 of the environment
REM    variable 'value'".

  if /I '%value:~0,13%'=='SASDAEMONHOST'   goto setdhost
  if /I '%value:~0,13%'=='SASDAEMONPORT'   goto setdport
  if /I '%value:~0,13%'=='SASCLIENTPORT'   goto setcport
  if /I '%value:~0,13%'=='SASARM4CORL8R'   goto setarmc
  if /I '%value:~0,13%'=='SASCOMMANDARG'   goto setargs
  if /I '%value:~0,13%'=='SASCHKRESTART'   goto setrestart
  if /I '%value:~0,13%'=='SASTKMPICPORT'   goto settkmpiport
  if /I '%value:~0,14%'=='SASBATCHWRKDIR'  goto setbatchwork
  if /I '%value:~0,14%'=='SASBATCHSASPGM'  goto setbatchpgm
  if /I '%value:~0,14%'=='SASWANTJOBINFO'  goto setjobinfo
  if /I '%value:~0,14%'=='SASTKMPICLIENT'  goto settkmpiclient
  if /I '%value:~0,14%'=='SASTKMPICTKLIB'  goto settkmpilib
  if /I '%value:~0,15%'=='SASMPIPROCESSES' goto setmpiprocs

REM -- If we fall through to here we either are running with
REM    a 9.1.3 client (which does not use name/value pairs)
REM    or there is a parameter we do not know about. Save the
REM    value to possibly use for SASDAEMONHOST,SASDAEMONPORT,
REM    and SASCLIENTPORT if they are not set.

  if '%tempvar1%'=='' goto setvar1
  if '%tempvar2%'=='' goto setvar2
  if '%tempvar3%'=='' goto setvar3

  goto nextopt

:setvar1
    set tempvar1=%value%
    goto nextopt

:setvar2
    set tempvar2=%value%
    goto nextopt

:setvar3
    set tempvar3=%value%
    goto nextopt


REM -- SASDAEMONHOST is the client host to connect back to
REM
REM    %value:~14% means "take the substring starting
REM    at position 14 to the end of the environment
REM    variable 'value'".

:setdhost
  set SASDAEMONHOST=%value:~14%
  goto nextopt


REM -- SASDAEMONPORT is the port on the client host
REM
REM    %value:~14% means "take the substring starting
REM    at position 14 to the end of the environment
REM    variable 'value'".

:setdport
  set SASDAEMONPORT=%value:~14%
  goto nextopt


REM -- SASCLIENTPORT is the communication context to use when talking
REM
REM    %value:~14% means "take the substring starting
REM    at position 14 to the end of the environment
REM    variable 'value'".

:setcport
  set SASCLIENTPORT=%value:~14%
  goto nextopt


REM -- SASARM4CORR is for setting the ARM4 correlator
REM
REM    %value:~14% means "take the substring starting
REM    at position 14 to the end of the environment
REM    variable 'value'".

:setarmc
  set SASARM4CORR=%value:~14%
  goto nextopt


REM -- SASGRIDWORKDIR is the directory to use for grid
REM    batch jobs
REM
REM    Some grid providers (like DataSynapse) like to put quotes
REM    around parameters which contain quotes. Since we are using
REM    the %value% variable which had quotes removed, we do not need
REM    to do anything fancy like the SASCOMMANDARG processing.

:setbatchwork
  set SASGRIDWORKDIR=%value:~15%
  goto nextopt


REM -- SASGRIDBATCHPGM is the program to run as a batch
REM    job. It is supposed to exist in the SASGRIDWORKDIR
REM    subdirectory
REM

:setbatchpgm
  set SASGRIDBATCHPGM=%value:~15%
  set RUNMODE=BATCH
  goto nextopt


REM -- SASWANTJOBINFO is options necessary to enable
REM    the output of job information files
REM

:setjobinfo
  set SASWANTJOBINFO=TRUE
  goto nextopt


REM -- SASCHKPTRESTART is options necessary to enable
REM    checkpoint restart
REM

:setrestart
  set SASCHKPTRESTART=TRUE
  goto nextopt


REM -- SASMPIPROCESSES is the number of MPI processes to start
REM

:setmpiprocs
  set SASMPIPROCESSES=%value:~16%
  set RUNMODE=MPI
  goto nextopt


REM -- SASTKMPICLIENT is the name of the TKMPI client host
REM

:settkmpiclient
  set SASTKMPICLIENT=%value:~15%
  set RUNMODE=MPI
  goto nextopt


REM -- SASTKMPICPORT is the port on the TKMPI client
REM

:settkmpiport
  set SASTKMPICPORT=%value:~14%
  set RUNMODE=MPI
  goto nextopt


REM -- SASTKMPICTKLIB is the TK library to run for the client
REM

:settkmpilib
  set SASTKMPICTKLIB=%value:~15%
  set RUNMODE=MPI
  goto nextopt


REM -- SASUSERARGS are the extra SAS parameters the user put in metadata
REM    Everything after the SASCOMMANDARG parm is expected to be sent
REM    as-is to the SAS command so we bypass the command processor
REM    parsing by grabbing %* and removing SASCOMMANDARG: and everything
REM    before it leaving only the SAS command arguments.
REM
REM    Some grid providers like to put quotes around parameters which
REM    causes problems so we check to see if "SASCOMMANDARG: ..."
REM    exists and remove the quotes before passing on the parameters
REM    We do this by changing '"SASCOMMANDARG' to 'QSASCOMMANDARG'
REM    and checking to see if anything changed. If nothing changed,
REM    the command string did not SASCOMMANDARG with a double quote
REM    in front of it. If it did change, SASCOMMANDARG did have a
REM    double quote in front of it and we need to remove it and the
REM    double quote at the end of the command line.

:setargs
  set SASUSERARGS=%*

  set TEMPUSERARGS=%SASUSERARGS:*"SASCOMMANDARG=QSASCOMMANDARG%

  if not '%TEMPUSERARGS:~0,14%'=='QSASCOMMANDARG' goto noquotes

  set SASUSERARGS=%TEMPUSERARGS%###
  set SASUSERARGS=%SASUSERARGS:"###=%

:noquotes
  set SASUSERARGS=%SASUSERARGS:*SASCOMMANDARG: =%
  goto exitloop


REM -- We are finished with this parameter, shift to set next
REM    parameter as %1 and go back to the parsing
:nextopt
  shift
  goto parseloop

REM -- We are finished with all parameters
:exitloop


REM /-----------------------------------------------------------
REM | If we get here and SASDAEMONHOST is not set, we must be on
REM | a 9.1.3 client. If that is the case, we expect the first
REM | three parameters to be SASDAEMONHOST, SASDAEMONPORT, and
REM | SASCLIENTPORT respectively and turn off the SSPI option.
REM \-----------------------------------------------------------
if not '%SASDAEMONHOST%'=='' goto skip913

  set SASDAEMONHOST=%tempvar1%
  set SASDAEMONPORT=%tempvar2%
  set SASCLIENTPORT=%tempvar3%

:skip913

REM /-----------------------------------------------------------
REM | If LSF has set the ARM4 parent correlator, use it
REM | as the SAS ARM4 instead of the one set by that parameters
REM |
REM | NOTE: To use ARM4, the -logconfigloc option needs
REM |       to be added to the command line to specify
REM |       the logging options.
REM \-----------------------------------------------------------
if not "%LSF_ARM4_PARENT%"=="" set SASARM4CORR=%LSF_ARM4_PARENT%

REM /-----------------------------------------------------------
REM | Check to see if the SAS Application Server directory has
REM | the script to set up the environment. If so, execute the
REM | script file.
REM \-----------------------------------------------------------
if exist "%SASGRIDCMDPATH%..\appservercontext_env.bat" (

  call "%SASGRIDCMDPATH%..\appservercontext_env.bat"
)

REM /-----------------------------------------------------------
REM | Check to see if the SAS Application Server directory has
REM | a configuration file to set options. If so, set the
REM | SASCFGPATH environment variable for the sas command
REM \-----------------------------------------------------------
if exist "%APPSERVER_ROOT%\sasv9.cfg" (

  set SASCFGPATH=-config "%APPSERVER_ROOT%\sasv9.cfg"
)

REM /-----------------------------------------------------------
REM | Check to see if the logging configuration file is present.
REM | If so, set the SASLOGCFGFILE environment variable for the
REM | sas command.
REM \-----------------------------------------------------------
if exist "%SASGRIDCMDPATH%\logconfig.xml" (

  set SASLOGCFGFILE=-logconfigloc "%SASGRIDCMDPATH%\logconfig.xml"
)

REM /-----------------------------------------------------------
REM | Try to run out of the user's temp directory if it exists.
REM | If it does not exist, the grid software should start the
REM | job in a writable directory since we cannot run in the
REM | AppServerContext directory since it may be a UNC path
REM | This is for submitting programs from XP to Vista/Win7
REM \-----------------------------------------------------------
if exist "%SYSTEMDRIVE%\Users\%USERNAME%\AppData\Local\Temp" (

  cd /D  "%SYSTEMDRIVE%\Users\%USERNAME%\AppData\Local\Temp"

) else (

  if exist "%SYSTEMDRIVE%\Documents and Settings\%USERNAME%\Local Settings\Temp" (

    cd /D  "%SYSTEMDRIVE%\Documents and Settings\%USERNAME%\Local Settings\Temp"
  )
)

REM /-----------------------------------------------------------
REM | If debugging, dump the environment variables
REM \-----------------------------------------------------------
if not '%SASGRIDSCRIPTDEBUG%'=='' (
    echo envVars in sasgrid
    set
)

REM /-----------------------------------------------------------
REM | If we are starting a CONNECT Server,
REM |
REM |   Start SAS at the installed location
REM |
REM |   %SASCFGPATH% is the path to the sasv9.cfg to use if
REM |    present in the application server directory.
REM |
REM |   %SASLOGCFGFILE% is the path to the logconfigloc file
REM |    to use if present in the GridServer directory.
REM |
REM |   '-dmr' starts the remote SAS session in SAS/CONNECT mode.
REM |
REM |   '-nologo' and '-noterminal' suppresses SAS messages and
REM |    prompts from remote SAS session.
REM |
REM |   '-nosyntaxcheck' prevents remote side from going into
REM |    syntax checking mode when a syntax error is encountered.
REM |
REM |   '-icon' forces window to come up iconized.
REM |
REM |   '-work .' makes sure work directory is created in the
REM |    current directory
REM |
REM |   '-sasuser work' makes sure the user has a sasuser area to
REM |    use just in case they have no logged onto the grid node
REM |    previously.
REM |
REM |   '-ipaddress' tells SAS to send this hosts IP address
REM |    back to the client. This is required for grid.
REM |
REM |   '-metaautoresources' tells SAS to load predefined
REM |    libraries that are associated with the SAS application
REM |    server.
REM |
REM |   %SASUSERARGS% are extra arguments the user defined
REM |    on the command in the metadata for SAS 9.2 and above
REM |
REM |   %SASUSERCONNECTOPTS% are extra arguments the user defined
REM |    in the grid_usermods.cfg file
REM \-----------------------------------------------------------
if not '%RUNMODE%'=='DMR' goto chkbatch

REM   /---------------------------------------------------------
REM   | Create the GRIDWORK temp directory for use to share
REM   | files with EG. Since mkdir does not distinguish between
REM   | a bad directory name or one that already exists in its
REM   | error code, we have to test for the directory first and 
REM   | fail everything if we cannot create it.
REM   \---------------------------------------------------------
  if exist "%SASGRIDSHAREDIR%\%USERNAME%" goto createdir

  echo Creating "%SASGRIDSHAREDIR%\%USERNAME%"
  mkdir "%SASGRIDSHAREDIR%\%USERNAME%"
  if "%ERRORLEVEL%"=="0" goto :createdir
  echo Invalid Grid share directory name "%SASGRIDSHAREDIR%"
  goto dirdone

:createdir
  set dirnum=0

:trynextdir
  set GRIDWORK=%SASGRIDSHAREDIR%\%USERNAME%\SASGRID_%COMPUTERNAME%_%DIRNUM%
  echo Creating "%GRIDWORK%"
  mkdir %GRIDWORK%
  if "%ERRORLEVEL%"=="0" goto dirdone
  set /a dirnum=%dirnum%+1
  goto trynextdir

:dirdone

REM   /---------------------------------------------------------
REM   | Start the CONNECT server
REM   \---------------------------------------------------------
  %SASEXEFILE% %SASCFGPATH% %SASLOGCFGFILE% -dmr -nologo -noterminal -nosyntaxcheck -icon -work . -sasuser work -ipaddress -metaautoresources %SASAPPSERVER% %SASUSERARGS% %SASUSERCONNECTOPTS%
  set SASRC=%ERRORLEVEL%

REM   /---------------------------------------------------------
REM   | Get rid of the GRIDWORK temporary directory
REM   \---------------------------------------------------------
  rmdir /s /q %GRIDWORK%


REM /-----------------------------------------------------------
REM | If we are starting SAS in batch mode,
REM \-----------------------------------------------------------
:chkbatch
if not '%RUNMODE%'=='BATCH' goto chkmpi

REM   /---------------------------------------------------------
REM   | Strip off enclosing single quotes from the work
REM   | directory
REM   \---------------------------------------------------------
  set SASGRIDWORKDIR=###%SASGRIDWORKDIR%###
  set SASGRIDWORKDIR=%SASGRIDWORKDIR:'###=%
  set SASGRIDWORKDIR=%SASGRIDWORKDIR:###'=%
  set SASGRIDWORKDIR=%SASGRIDWORKDIR:###=%

REM   /---------------------------------------------------------
REM   | Set the program to run before stripping off the
REM   | extension if it has one. Since the program is in the
REM   | work directory, we must prepend the name with the
REM   | grid work directory path
REM   \---------------------------------------------------------
  set SASPGM="%SASGRIDWORKDIR%\%SASGRIDBATCHPGM%"

REM   /---------------------------------------------------------
REM   | Strip off the extension from the program path
REM   \---------------------------------------------------------
  set SASGRIDBATCHPGM=%SASGRIDBATCHPGM%###
  set SASGRIDBATCHPGM=%SASGRIDBATCHPGM:.SAS###=%
  set SASGRIDBATCHPGM=%SASGRIDBATCHPGM:.sas###=%
  set SASGRIDBATCHPGM=%SASGRIDBATCHPGM:###=%

REM   /---------------------------------------------------------
REM   | Set the log and list output names. Since the files need
REM   | to be stored in the work directory, we must prepend the
REM   | name with the grid work directory path
REM   \---------------------------------------------------------
  set SASLOG=-LOG "%SASGRIDWORKDIR%\%SASGRIDBATCHPGM%.log"
  set SASLST=-PRINT "%SASGRIDWORKDIR%\%SASGRIDBATCHPGM%.lst"

REM   /---------------------------------------------------------
REM   | Set the parameter that will allow the user to code
REM   | a libname or filename statement using GRIDWORK
REM   \---------------------------------------------------------
  set SETGRIDWORK=-set GRIDWORK '%SASGRIDWORKDIR%'

  if '%SASCHKPTRESTART%'=='TRUE' (
    set SASCHKPTRESTART=-STEPCHKPT -NOWORKINIT -NOWORKTERM -STEPRESTART
    set SASWRK=-WORK '%SASGRIDWORKDIR%\WORK'
  ) else (
    set SASCHKPTRESTART=
    set SASWRK=-WORK .
  )

REM   /---------------------------------------------------------
REM   | Output the host we are running on to host.<hostname>.
REM   |  This file's creation time is the start time of the job
REM   | Delete any existing time.end just in case this job is
REM   |  being restarted.
REM   \---------------------------------------------------------
  if '%SASWANTJOBINFO%'=='TRUE' (

    echo Host=%COMPUTERNAME% > "%SASGRIDWORKDIR%\host.%COMPUTERNAME%
    del "%SASGRIDWORKDIR%\time.end"
  )

REM   /---------------------------------------------------------
REM   | Run SAS pointing to the program in the GRIDWORKDIR and
REM   | logging and listing files into GRIDWORKDIR
REM   \---------------------------------------------------------
  %SASEXEFILE% %SASCFGPATH% %SASLOGCFGFILE% -nologo -noterminal -icon %SASWRK% -sasuser work %SASCHKPTRESTART% -sysin %SASPGM% %SASLOG% %SASLST% %SETGRIDWORK% %SASUSERARGS%
  set SASRC=%ERRORLEVEL%

REM   /---------------------------------------------------------
REM   | When complete, output the time.end file. This file's
REM   |  creation time is the end time of the job.
REM   \---------------------------------------------------------
  if '%SASWANTJOBINFO%'=='TRUE' (

    echo EndTime=%DATE% %TIME% > "%SASGRIDWORKDIR%\time.end
  )

REM /-----------------------------------------------------------
REM | If we are starting MPI
REM \-----------------------------------------------------------
:chkmpi
if not '%RUNMODE%'=='MPI' goto done

REM   /---------------------------------------------------------
REM   | Run MPI starting up the proper TK library with the
REM   | proper options
REM   \---------------------------------------------------------
  "%SASGRIDCMDPATH%\runmpi.cmd" -np %SASMPIPROCESSES% "%SASGRIDCMDPATH%\runtkmpi.cmd" %SASTKMPICLIENT% %SASTKMPICPORT% %SASTKMPICTKLIB%
  set SASRC=%ERRORLEVEL%


REM /-----------------------------------------------------------
REM | All done...
REM \-----------------------------------------------------------
:done

exit /b %SASRC%
